var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useCallback, useEffect, useState } from 'react';
import { ActionType } from '../../enums';
import { addEscEnterKeyEffect } from '../../Utils/EffectUtils';
import { getValidationValue } from '../../Utils/Validation';
import CellEditorDataType from '../CellEditorDataType/CellEditorDataType';
import CellEditorValidationMessage from '../CellEditorValidationMessage/CellEditorValidationMessage';
var CellEditorState = function (props) {
    var column = props.column, field = props.field, key = props.column.key, rowData = props.rowData, rowKeyField = props.rowKeyField, dispatch = props.dispatch;
    var _a = useState(rowData), value = _a[0], changeValue = _a[1];
    var validationValue = getValidationValue(value, field, column.validation);
    var onValueStateChange = function (rowValue) {
        changeValue(rowValue);
    };
    var close = useCallback(function () {
        var cell = { columnKey: column.key, rowKey: rowData[rowKeyField] };
        dispatch(ActionType.CloseEditor, { cell: cell });
    }, [dispatch, column, rowData, rowKeyField]);
    var closeHandler = useCallback(function () {
        var _a;
        if (!validationValue) {
            if (rowData[key] !== value[key]) {
                dispatch(ActionType.ChangeRowData, { newValue: __assign(__assign({}, rowData), (_a = {}, _a[key] = value[key], _a)) });
            }
            close();
        }
    }, [validationValue, dispatch, close, value, key, rowData]);
    useEffect(function () {
        return addEscEnterKeyEffect(close, closeHandler);
    }, [close, closeHandler]);
    var dispatchHandler = function (action, actionData) {
        if (action === ActionType.CloseEditor) {
            closeHandler();
        }
        else if (action === ActionType.ChangeRowData) {
            onValueStateChange(actionData.newValue);
        }
        else {
            dispatch(action, actionData);
        }
    };
    var stateProps = __assign(__assign({}, props), {
        dispatch: dispatchHandler,
        rowData: value,
    });
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: "ka-cell-editor " + (validationValue ? 'ka-cell-editor-validation-error' : '') },
            React.createElement(CellEditorDataType, __assign({}, stateProps)),
            validationValue && React.createElement(CellEditorValidationMessage, { message: validationValue }))));
};
export default CellEditorState;