import defaultOptions from '../defaultOptions';
import { DataType, FilterOperatorName } from '../enums';
import { getField } from './ColumnUtils';
import { isEmpty } from './CommonUtils';
export var getRowEditableCells = function (rowKeyValue, editableCells) {
    return editableCells.filter(function (c) { return c.rowKey === rowKeyValue; });
};
export var searchData = function (columns, data, searchText) {
    return columns.reduce(function (initialData, c) {
        var filterFunction = function (item) {
            return c.search ? c.search(searchText, item, c) : initialData.indexOf(item) < 0
                && item[getField(c)].toString().toLowerCase().includes(searchText.toLowerCase());
        };
        return initialData.concat(data.filter(filterFunction));
    }, []);
};
export var filterData = function (data, columns) {
    return columns.reduce(function (initialData, column) {
        if (isEmpty(column.filterRowValue)) {
            return initialData;
        }
        var filterRowOperator = column.filterRowOperator
            || getDefaultOperatorForType(column.dataType || defaultOptions.columnDataType);
        var filterOperator = predefinedFilterOperators.find(function (fo) { return filterRowOperator === fo.name; });
        if (!filterOperator) {
            throw new Error("'" + column.filterRowOperator + "' has not found in predefinedFilterOperators array, available operators: " + predefinedFilterOperators.map(function (o) { return o.name; }).join(', '));
        }
        var compare = filterOperator.compare;
        return initialData.filter(function (d) {
            var fieldValue = d[getField(column)];
            var conditionValue = column.filterRowValue;
            if (column.dataType === DataType.Date) {
                fieldValue = new Date(fieldValue).setHours(0, 0, 0, 0);
                conditionValue = new Date(conditionValue).setHours(0, 0, 0, 0);
            }
            return compare(fieldValue, conditionValue);
        });
    }, data);
};
export var getDefaultOperatorForType = function (type) {
    var filterOperator = predefinedFilterOperators.find(function (o) { return o.defaultForTypes && o.defaultForTypes.includes(type); });
    return (filterOperator && filterOperator.name) || '=';
};
export var predefinedFilterOperators = [{
        compare: function (fieldValue, conditionValue) {
            return fieldValue === conditionValue;
        },
        defaultForTypes: [DataType.Boolean, DataType.Number, DataType.Date],
        name: FilterOperatorName.Equal,
    }, {
        compare: function (fieldValue, conditionValue) {
            return fieldValue > conditionValue;
        },
        name: FilterOperatorName.MoreThan,
    }, {
        compare: function (fieldValue, conditionValue) {
            return fieldValue < conditionValue;
        },
        name: FilterOperatorName.LessThan,
    }, {
        compare: function (fieldValue, conditionValue) {
            return fieldValue >= conditionValue;
        },
        name: FilterOperatorName.MoreThanOrEqual,
    }, {
        compare: function (fieldValue, conditionValue) {
            return fieldValue <= conditionValue;
        },
        name: FilterOperatorName.LessThanOrEqual,
    }, {
        compare: function (fieldValue, conditionValue) {
            return fieldValue.toString().toLowerCase().includes(conditionValue.toString().toLowerCase());
        },
        defaultForTypes: [DataType.String],
        name: FilterOperatorName.Contains,
    }];